hold on;close all;

import casadi.*

% -- Start of computations --

a = 10.0;

n_p = 5;

x = MX.sym('x', n_p);
y = MX.sym('y', n_p);

R = MX.sym('R', 1);


r = [R R R 2*R 2*R];

g = {};
g_min = [];
g_max = [];

for i = 1:n_p

    g = {g{:} x(i) - r(i)};
    g_min = [g_min; 0];
    g_max = [g_max; Inf];

    g = {g{:} x(i) + r(i)};
    g_min = [g_min; -Inf];
    g_max = [g_max; a];
    
    g = {g{:} y(i) - r(i)};
    g_min = [g_min; 0];
    g_max = [g_max; Inf];

    g = {g{:} y(i) + r(i)};
    g_min = [g_min; -Inf];
    g_max = [g_max; a];
    
end
    
for i = 1:n_p

    for j = i+1:n_p

        g = {g{:} (x(i) - x(j)).^2 + (y(i) - y(j)).^2 - (r(i) + r(j)).^2};
        g_min = [g_min; 0];
        g_max = [g_max; Inf];

    end
    
end
        
g = vertcat(g{:});

x_init = [1; 3; 8; 7; 3];
y_init = [1; 3; 8; 3; 7];
R_init = 1.0;


plot_plate(x_init, y_init, R, r, R_init, a, n_p)


V = vertcat(x, y, R);

nlp = struct('x', V, 'f', -R, 'g', g);

solver = nlpsol('solver', 'ipopt', nlp);

V_init = vertcat(x_init, y_init, R_init);

x_min = zeros(n_p, 1);
x_max = a * ones(n_p, 1);

y_min = zeros(n_p, 1);
y_max = a * ones(n_p, 1);

R_min = 0.0;
R_max = Inf;

V_min = vertcat(x_min, y_min, R_min);
V_max = vertcat(x_max, y_max, R_max);

solution = solver('x0', V_init, 'lbx', V_min, 'ubx',  V_max, ...
    'lbg', g_min, 'ubg', g_max);

V_opt = solution.x;

x_opt = full(V_opt(1:n_p));
y_opt = full(V_opt(n_p+1:end-1));
R_opt = full(V_opt(end));


plot_plate(x_opt, y_opt, R, r, R_opt, a, n_p)
